@@ -45,12 +45,12 @@ AdjustVal(Fixed* pv, Fixed l1, Fixed l2, Fixed dist, Fixed d, bool hFlg)
         q = (float)dist;
         q = q * q;
     }
-    v = (float)((1000.0 * r1 * r2) / (q * q));
+    v = (float)((1000.0f * r1 * r2) / (q * q));
     if (d <= (hFlg ? gHBigDist : gVBigDist))
         goto done;
     acfixtopflt(d, &rd);
     q = (hFlg ? gHBigDistR : gVBigDistR) / rd; /* 0 < q < 1.0 */
-    if (q <= 0.5) {
+    if (q <= 0.5f) {
         v = 0.0;
         goto done;
     }
@@ -61,7 +61,7 @@ AdjustVal(Fixed* pv, Fixed l1, Fixed l2, Fixed dist, Fixed d, bool hFlg)
 done:
     if (v > gMaxVal)
         v = gMaxVal;
-    else if (v > 0.0 && v < gMinVal)
+    else if (v > 0.0f && v < gMinVal)
         v = gMinVal;
     *pv = acpflttofix(&v);
 }
@@ -70,7 +70,7 @@ static Fixed
 CalcOverlapDist(Fixed d, Fixed overlaplen, Fixed minlen)
 {
     float r = (float)d, ro = (float)overlaplen, rm = (float)minlen;
-    r = r * ((float)(1.0 + 0.4 * (1.0 - ro / rm)));
+    r = r * ((float)(1.0f + 0.4f * (1.0f - ro / rm)));
     d = (Fixed)r;
     return d;
 }
@@ -94,12 +94,11 @@ CalcOverlapDist(Fixed d, Fixed overlaplen, Fixed minlen)
  */
 
 static void
-EvalHPair(PClrSeg botSeg, PClrSeg topSeg, Fixed* pspc, Fixed* pv)
+EvalHPair(HintSeg* botSeg, HintSeg* topSeg, Fixed* pspc, Fixed* pv)
 {
-    Fixed brght, blft, bloc, tloc, trght, tlft, ldst, rdst;
-    Fixed mndist, dist, dx, dy, minlen, overlaplen;
+    Fixed brght, blft, bloc, tloc, trght, tlft;
+    Fixed mndist, dist, dy;
     bool inBotBand, inTopBand;
-    int i;
     *pspc = 0;
     brght = botSeg->sMax;
     blft = botSeg->sMin;
@@ -122,28 +121,23 @@ EvalHPair(PClrSeg botSeg, PClrSeg topSeg, Fixed* pspc, Fixed* pv)
         *pspc = FixInt(2);
     /* left is always < right */
     if ((tlft <= brght) && (trght >= blft)) { /* overlap */
-        overlaplen = NUMMIN(trght, brght) - NUMMAX(tlft, blft);
-        minlen = NUMMIN(trght - tlft, brght - blft);
+        Fixed overlaplen = NUMMIN(trght, brght) - NUMMAX(tlft, blft);
+        Fixed minlen = NUMMIN(trght - tlft, brght - blft);
         if (minlen == overlaplen)
             dist = dy;
         else
             dist = CalcOverlapDist(dy, overlaplen, minlen);
     } else { /* no overlap; take closer ends */
-        ldst = abs(tlft - brght);
-        rdst = abs(trght - blft);
+        Fixed dx;
+        Fixed ldst = abs(tlft - brght);
+        Fixed rdst = abs(trght - blft);
         dx = NUMMIN(ldst, rdst);
         dist = GapDist(dx);
-        dist +=
-          (7 * dy) /
-          5;              /* extra penalty for nonoverlap
-                                                                                * changed
-                           * from 7/5 to 12/5 for Perpetua/Regular/
-                                                                                * n, r ,m and
-                           * other lowercase serifs;
-                                                                                * undid
-                           * change for Berthold/AkzidenzGrotesk 9/16/91;
-                                                                                * this did
-                           * not make Perpetua any worse. */
+        /* extra penalty for nonoverlap changed from 7/5 to 12/5 for
+         * Perpetua/Regular/ n, r ,m and other lowercase serifs; undid change
+         * for Berthold/AkzidenzGrotesk 9/16/91; this did not make Perpetua any
+         * worse. */
+        dist += (7 * dy) / 5;
         DEBUG_ROUND(dist) /* DEBUG 8 BIT */
         if (dx > dy)
             dist *= dx / dy;
@@ -151,8 +145,8 @@ EvalHPair(PClrSeg botSeg, PClrSeg topSeg, Fixed* pspc, Fixed* pv)
     mndist = FixTwoMul(gMinDist);
     dist = NUMMAX(dist, mndist);
     if (gNumHStems > 0) {
-        Fixed w = idtfmy(dy);
-        w = abs(w);
+        int i;
+        Fixed w = abs(dy);
         for (i = 0; i < gNumHStems; i++)
             if (w == gHStems[i]) {
                 *pspc += FixOne;
@@ -163,11 +157,11 @@ EvalHPair(PClrSeg botSeg, PClrSeg topSeg, Fixed* pspc, Fixed* pv)
 }
 
 static void
-HStemMiss(PClrSeg botSeg, PClrSeg topSeg)
+HStemMiss(HintSeg* botSeg, HintSeg* topSeg)
 {
     Fixed brght, blft, bloc, tloc, trght, tlft;
-    Fixed mndist, dist, dy, minlen, overlaplen;
-    Fixed b, t, diff, minDiff, minW, w, sw;
+    Fixed mndist, dist, dy;
+    Fixed b, t, minDiff, minW, w;
     int i;
     if (gNumHStems == 0)
         return;
@@ -182,8 +176,8 @@ HStemMiss(PClrSeg botSeg, PClrSeg topSeg)
         return;
     /* left is always < right */
     if ((tlft <= brght) && (trght >= blft)) { /* overlap */
-        overlaplen = NUMMIN(trght, brght) - NUMMAX(tlft, blft);
-        minlen = NUMMIN(trght - tlft, brght - blft);
+        Fixed overlaplen = NUMMIN(trght, brght) - NUMMAX(tlft, blft);
+        Fixed minlen = NUMMIN(trght - tlft, brght - blft);
         if (minlen == overlaplen)
             dist = dy;
         else
@@ -195,16 +189,16 @@ HStemMiss(PClrSeg botSeg, PClrSeg topSeg)
         return;
     minDiff = FixInt(1000);
     minW = 0;
-    b = itfmy(bloc);
-    t = itfmy(tloc);
+    b = -bloc;
+    t = -tloc;
     w = t - b;
     /* don't check ghost bands for near misses */
     if (((w = t - b) == botGhst) || (w == topGhst))
         return;
     w = abs(w);
     for (i = 0; i < gNumHStems; i++) {
-        sw = gHStems[i];
-        diff = abs(sw - w);
+        Fixed sw = gHStems[i];
+        Fixed diff = abs(sw - w);
         if (diff == 0)
             return;
         if (diff < minDiff) {
@@ -219,12 +213,11 @@ HStemMiss(PClrSeg botSeg, PClrSeg topSeg)
 }
 
 static void
-EvalVPair(PClrSeg leftSeg, PClrSeg rightSeg, Fixed* pspc, Fixed* pv)
+EvalVPair(HintSeg* leftSeg, HintSeg* rightSeg, Fixed* pspc, Fixed* pv)
 {
-    Fixed ltop, lbot, lloc, rloc, rtop, rbot, tdst, bdst;
-    Fixed mndist, dx, dy, dist, overlaplen, minlen;
+    Fixed ltop, lbot, lloc, rloc, rtop, rbot;
+    Fixed mndist, dx, dist;
     Fixed bonus, lbonus, rbonus;
-    int i;
     *pspc = 0;
     ltop = leftSeg->sMax;
     lbot = leftSeg->sMin;
@@ -239,16 +232,16 @@ EvalVPair(PClrSeg leftSeg, PClrSeg rightSeg, Fixed* pspc, Fixed* pv)
     }
     /* top is always > bot, independent of YgoesUp */
     if ((ltop >= rbot) && (lbot <= rtop)) { /* overlap */
-        overlaplen = NUMMIN(ltop, rtop) - NUMMAX(lbot, rbot);
-        minlen = NUMMIN(ltop - lbot, rtop - rbot);
+        Fixed overlaplen = NUMMIN(ltop, rtop) - NUMMAX(lbot, rbot);
+        Fixed minlen = NUMMIN(ltop - lbot, rtop - rbot);
         if (minlen == overlaplen)
             dist = dx;
         else
             dist = CalcOverlapDist(dx, overlaplen, minlen);
     } else { /* no overlap; take closer ends */
-        tdst = abs(ltop - rbot);
-        bdst = abs(lbot - rtop);
-        dy = NUMMIN(tdst, bdst);
+        Fixed tdst = abs(ltop - rbot);
+        Fixed bdst = abs(lbot - rtop);
+        Fixed dy = NUMMIN(tdst, bdst);
         dist = (7 * dx) / 5 + GapDist(dy); /* extra penalty for nonoverlap */
         DEBUG_ROUND(dist)                  /* DEBUG 8 BIT */
         if (dy > dx)
@@ -261,8 +254,8 @@ EvalVPair(PClrSeg leftSeg, PClrSeg rightSeg, Fixed* pspc, Fixed* pv)
     bonus = NUMMIN(lbonus, rbonus);
     *pspc = (bonus > 0) ? FixInt(2) : 0; /* this is for sol-eol characters */
     if (gNumVStems > 0) {
-        Fixed w = idtfmx(dx);
-        w = abs(w);
+        int i;
+        Fixed w = abs(dx);
         for (i = 0; i < gNumVStems; i++)
             if (w == gVStems[i]) {
                 *pspc = *pspc + FixOne;
@@ -273,11 +266,10 @@ EvalVPair(PClrSeg leftSeg, PClrSeg rightSeg, Fixed* pspc, Fixed* pv)
 }
 
 static void
-VStemMiss(PClrSeg leftSeg, PClrSeg rightSeg)
+VStemMiss(HintSeg* leftSeg, HintSeg* rightSeg)
 {
     Fixed ltop, lbot, lloc, rloc, rtop, rbot;
-    Fixed mndist, dx, dist, overlaplen, minlen;
-    Fixed l, r, diff, minDiff, minW, w, sw;
+    Fixed dx, l, r, minDiff, minW, w;
     int i;
     if (gNumVStems == 0)
         return;
@@ -291,25 +283,17 @@ VStemMiss(PClrSeg leftSeg, PClrSeg rightSeg)
     if (dx < gMinDist)
         return;
     /* top is always > bot, independent of YgoesUp */
-    if ((ltop >= rbot) && (lbot <= rtop)) { /* overlap */
-        overlaplen = NUMMIN(ltop, rtop) - NUMMAX(lbot, rbot);
-        minlen = NUMMIN(ltop - lbot, rtop - rbot);
-        if (minlen == overlaplen)
-            dist = dx;
-        else
-            dist = CalcOverlapDist(dx, overlaplen, minlen);
-    } else
+    if ((ltop < rbot) || (lbot > rtop))  /* does not overlap */
         return;
-    mndist = FixTwoMul(gMinDist);
-    dist = NUMMAX(dist, mndist);
-    l = itfmx(lloc);
-    r = itfmx(rloc);
+
+    l = lloc;
+    r = rloc;
     w = abs(r - l);
     minDiff = FixInt(1000);
     minW = 0;
     for (i = 0; i < gNumVStems; i++) {
-        sw = gVStems[i];
-        diff = abs(sw - w);
+        Fixed sw = gVStems[i];
+        Fixed diff = abs(sw - w);
         if (diff < minDiff) {
             minDiff = diff;
             minW = sw;
@@ -325,11 +309,11 @@ VStemMiss(PClrSeg leftSeg, PClrSeg rightSeg)
 }
 
 static void
-InsertVValue(Fixed lft, Fixed rght, Fixed val, Fixed spc, PClrSeg lSeg,
-             PClrSeg rSeg)
+InsertVValue(Fixed lft, Fixed rght, Fixed val, Fixed spc, HintSeg* lSeg,
+             HintSeg* rSeg)
 {
-    PClrVal item, vlist, vprev;
-    item = (PClrVal)Alloc(sizeof(ClrVal));
+    HintVal *item, *vlist, *vprev;
+    item = (HintVal*)Alloc(sizeof(HintVal));
     item->vVal = val;
     item->initVal = val;
     item->vLoc1 = lft;
@@ -357,15 +341,14 @@ InsertVValue(Fixed lft, Fixed rght, Fixed val, Fixed spc, PClrSeg lSeg,
     else
         vprev->vNxt = item;
     item->vNxt = vlist;
-    if (gShowClrInfo && gShowVs)
-        ReportAddVVal(item);
+    ReportAddVVal(item);
 }
 
 #define LePruneValue(val) ((val) < FixOne && ((val) << 10) <= gPruneValue)
 
 static void
-AddVValue(Fixed lft, Fixed rght, Fixed val, Fixed spc, PClrSeg lSeg,
-          PClrSeg rSeg)
+AddVValue(Fixed lft, Fixed rght, Fixed val, Fixed spc, HintSeg* lSeg,
+          HintSeg* rSeg)
 {
     if (val == 0)
         return;
@@ -379,14 +362,16 @@ AddVValue(Fixed lft, Fixed rght, Fixed val, Fixed spc, PClrSeg lSeg,
             !CheckBBoxes(lSeg->sElt, rSeg->sElt))
             return;
     }
+    if (rSeg == NULL)
+        return;
     InsertVValue(lft, rght, val, spc, lSeg, rSeg);
 }
 
 static void
-InsertHValue(Fixed bot, Fixed top, Fixed val, Fixed spc, PClrSeg bSeg,
-             PClrSeg tSeg, bool ghst)
+InsertHValue(Fixed bot, Fixed top, Fixed val, Fixed spc, HintSeg* bSeg,
+             HintSeg* tSeg, bool ghst)
 {
-    PClrVal item, vlist, vprev, vl;
+    HintVal *item, *vlist, *vprev, *vl;
     vlist = gValList;
     vprev = NULL;
     while (vlist != NULL) {
@@ -411,7 +396,7 @@ InsertHValue(Fixed bot, Fixed top, Fixed val, Fixed spc, PClrSeg bSeg,
             return;
         vl = vl->vNxt;
     }
-    item = (PClrVal)Alloc(sizeof(ClrVal));
+    item = (HintVal*)Alloc(sizeof(HintVal));
     item->vVal = val;
     item->initVal = val;
     item->vSpc = spc;
@@ -425,13 +410,12 @@ InsertHValue(Fixed bot, Fixed top, Fixed val, Fixed spc, PClrSeg bSeg,
     else
         vprev->vNxt = item;
     item->vNxt = vlist;
-    if (gShowClrInfo && gShowHs)
-        ReportAddHVal(item);
+    ReportAddHVal(item);
 }
 
 static void
-AddHValue(Fixed bot, Fixed top, Fixed val, Fixed spc, PClrSeg bSeg,
-          PClrSeg tSeg)
+AddHValue(Fixed bot, Fixed top, Fixed val, Fixed spc, HintSeg* bSeg,
+          HintSeg* tSeg)
 {
     bool ghst;
     if (val == 0)
@@ -462,7 +446,7 @@ CombVals(Fixed v1, Fixed v2)
 {
     int32_t i;
     float r1, r2;
-    float x, a, xx;
+    float x, a, xx = 0;
     acfixtopflt(v1, &r1);
     acfixtopflt(v2, &r2);
     /* home brew sqrt */
@@ -470,7 +454,7 @@ CombVals(Fixed v1, Fixed v2)
     x = a;
     for (i = 0; i < 16; i++) {
         xx = ((float)0.5) * (x + a / x);
-        if (i >= 8 && mfabs(xx - x) <= mfabs(xx) * 0.0000001)
+        if (i >= 8 && mfabs(xx - x) <= mfabs(xx) * 0.0000001f)
             break;
         x = xx;
     }
@@ -485,17 +469,13 @@ CombVals(Fixed v1, Fixed v2)
 static void
 CombineValues(void)
 { /* works for both H and V */
-    PClrVal vlist, v1;
-    Fixed loc1, loc2;
-    Fixed val;
-    bool match;
-    vlist = gValList;
+    HintVal* vlist = gValList;
     while (vlist != NULL) {
-        v1 = vlist->vNxt;
-        loc1 = vlist->vLoc1;
-        loc2 = vlist->vLoc2;
-        val = vlist->vVal;
-        match = false;
+        HintVal* v1 = vlist->vNxt;
+        Fixed loc1 = vlist->vLoc1;
+        Fixed loc2 = vlist->vLoc2;
+        Fixed val = vlist->vVal;
+        bool match = false;
         while (v1 != NULL && v1->vLoc1 == loc1 && v1->vLoc2 == loc2) {
             if (v1->vGhst)
                 val = v1->vVal;
@@ -518,7 +498,7 @@ CombineValues(void)
 void
 EvalV(void)
 {
-    PClrSeg lList, rList;
+    HintSeg *lList, *rList;
     Fixed lft, rght;
     Fixed val, spc;
     gValList = NULL;
@@ -543,7 +523,7 @@ EvalV(void)
 void
 EvalH(void)
 {
-    PClrSeg bList, tList, lst, ghostSeg;
+    HintSeg *bList, *tList, *lst, *ghostSeg;
     Fixed lstLoc, tempLoc, cntr;
     Fixed val, spc;
     gValList = NULL;
@@ -554,7 +534,7 @@ EvalH(void)
             Fixed bot, top;
             bot = bList->sLoc;
             top = tList->sLoc;
-            if ((bot < top && gYgoesUp) || (bot > top && !gYgoesUp)) {
+            if (bot > top) {
                 EvalHPair(bList, tList, &spc, &val);
                 HStemMiss(bList, tList);
                 AddHValue(bot, top, val, spc, bList, tList);
@@ -563,7 +543,7 @@ EvalH(void)
         }
         bList = bList->sNxt;
     }
-    ghostSeg = (PClrSeg)Alloc(sizeof(ClrSeg));
+    ghostSeg = (HintSeg*)Alloc(sizeof(HintSeg));
     ghostSeg->sType = sGHOST;
     ghostSeg->sElt = NULL;
     if (gLenBotBands < 2 && gLenTopBands < 2)
@@ -572,11 +552,7 @@ EvalH(void)
     while (lst != NULL) {
         lstLoc = lst->sLoc;
         if (InBlueBand(lstLoc, gLenBotBands, gBotBands)) {
-            tempLoc = lstLoc;
-            if (gYgoesUp)
-                tempLoc += gGhostWidth;
-            else
-                tempLoc -= gGhostWidth;
+            tempLoc = lstLoc - gGhostWidth;
             ghostSeg->sLoc = tempLoc;
             cntr = (lst->sMax + lst->sMin) / 2;
             ghostSeg->sMax = cntr + gGhostLength / 2;
@@ -593,11 +569,7 @@ EvalH(void)
     while (lst != NULL) {
         lstLoc = lst->sLoc;
         if (InBlueBand(lstLoc, gLenTopBands, gTopBands)) {
-            tempLoc = lstLoc;
-            if (!gYgoesUp)
-                tempLoc += gGhostWidth;
-            else
-                tempLoc -= gGhostWidth;
+            tempLoc = lstLoc + gGhostWidth;
             ghostSeg->sLoc = tempLoc;
             cntr = (lst->sMin + lst->sMax) / 2;
             ghostSeg->sMax = cntr + gGhostLength / 2;
