@@ -12,7 +12,7 @@
 #include "ac.h"
 #include "bbox.h"
 
-static PSegLnkLst Hlnks, Vlnks;
+static SegLnkLst *Hlnks, *Vlnks;
 static int32_t cpFrom, cpTo;
 
 void
@@ -29,14 +29,14 @@ InitGen(int32_t reason)
 }
 
 static void
-LinkSegment(PPathElt e, bool Hflg, PClrSeg seg)
+LinkSegment(PathElt* e, bool Hflg, HintSeg* seg)
 {
-    PSegLnk newlnk;
-    PSegLnkLst newlst, globlst;
-    newlnk = (PSegLnk)Alloc(sizeof(SegLnk));
+    SegLnk* newlnk;
+    SegLnkLst *newlst, *globlst;
+    newlnk = (SegLnk*)Alloc(sizeof(SegLnk));
     newlnk->seg = seg;
-    newlst = (PSegLnkLst)Alloc(sizeof(SegLnkLst));
-    globlst = (PSegLnkLst)Alloc(sizeof(SegLnkLst));
+    newlst = (SegLnkLst*)Alloc(sizeof(SegLnkLst));
+    globlst = (SegLnkLst*)Alloc(sizeof(SegLnkLst));
     globlst->lnk = newlnk;
     newlst->lnk = newlnk;
     if (Hflg) {
@@ -53,11 +53,11 @@ LinkSegment(PPathElt e, bool Hflg, PClrSeg seg)
 }
 
 static void
-CopySegmentLink(PPathElt e1, PPathElt e2, bool Hflg)
+CopySegmentLink(PathElt* e1, PathElt* e2, bool Hflg)
 {
     /* copy reference to first link from e1 to e2 */
-    PSegLnkLst newlst;
-    newlst = (PSegLnkLst)Alloc(sizeof(SegLnkLst));
+    SegLnkLst* newlst;
+    newlst = (SegLnkLst*)Alloc(sizeof(SegLnkLst));
     if (Hflg) {
         newlst->lnk = e1->Hs->lnk;
         newlst->next = e2->Hs;
@@ -71,11 +71,11 @@ CopySegmentLink(PPathElt e1, PPathElt e2, bool Hflg)
 
 static void
 AddSegment(Fixed from, Fixed to, Fixed loc, int32_t lftLstNm, int32_t rghtLstNm,
-           PPathElt e1, PPathElt e2, bool Hflg, int32_t typ)
+           PathElt* e1, PathElt* e2, bool Hflg, int32_t typ)
 {
-    PClrSeg seg, segList, prevSeg;
+    HintSeg *seg, *segList, *prevSeg;
     int32_t segNm;
-    seg = (PClrSeg)Alloc(sizeof(ClrSeg));
+    seg = (HintSeg*)Alloc(sizeof(HintSeg));
     seg->sLoc = loc;
     if (from > to) {
         seg->sMax = from;
@@ -95,7 +95,8 @@ AddSegment(Fixed from, Fixed to, Fixed loc, int32_t lftLstNm, int32_t rghtLstNm,
     if (e2 != NULL) {
         if (e2->type == CLOSEPATH)
             e2 = GetDest(e2);
-        CopySegmentLink(e1, e2, Hflg);
+        if (e1 != NULL)
+            CopySegmentLink(e1, e2, Hflg);
         if (e1 == NULL || e2 == e1->prev)
             seg->sElt = e2;
     }
@@ -125,23 +126,22 @@ AddSegment(Fixed from, Fixed to, Fixed loc, int32_t lftLstNm, int32_t rghtLstNm,
 }
 
 void
-AddVSegment(Fixed from, Fixed to, Fixed loc, PPathElt p1, PPathElt p2,
+AddVSegment(Fixed from, Fixed to, Fixed loc, PathElt* p1, PathElt* p2,
             int32_t typ, int32_t i)
 {
-    if (gDebug)
-        ReportAddVSeg(from, to, loc, i);
-    if (gYgoesUp)
-        AddSegment(from, to, loc, 0, 1, p1, p2, false, typ);
-    else
-        AddSegment(from, to, loc, 1, 0, p1, p2, false, typ);
+    LogMsg(LOGDEBUG, OK, "add vseg %g %g to %g %g %d", FixToDbl(loc),
+           FixToDbl(-from), FixToDbl(loc), FixToDbl(-to), i);
+
+    AddSegment(from, to, loc, 1, 0, p1, p2, false, typ);
 }
 
 void
-AddHSegment(Fixed from, Fixed to, Fixed loc, PPathElt p1, PPathElt p2,
+AddHSegment(Fixed from, Fixed to, Fixed loc, PathElt* p1, PathElt* p2,
             int32_t typ, int32_t i)
 {
-    if (gDebug)
-        ReportAddHSeg(from, to, loc, i);
+    LogMsg(LOGDEBUG, OK, "add hseg %g %g to %g %g %d", FixToDbl(from),
+           FixToDbl(-loc), FixToDbl(to), FixToDbl(-loc), i);
+
     AddSegment(from, to, loc, 2, 3, p1, p2, true, typ);
 }
 
@@ -189,43 +189,40 @@ static bool
 IsCCW(Fixed x0, Fixed y0, Fixed x1, Fixed y1, Fixed x2, Fixed y2)
 {
     /* returns true if (x0,y0) -> (x1,y1) -> (x2,y2) is counter clockwise
-         in character space */
+         in glyph space */
     int32_t dx0, dy0, dx1, dy1;
     bool ccw;
     dx0 = FRound(x1 - x0);
-    dy0 = FRound(y1 - y0);
+    dy0 = -FRound(y1 - y0);
     dx1 = FRound(x2 - x1);
-    dy1 = FRound(y2 - y1);
-    if (!gYgoesUp) {
-        dy0 = -dy0;
-        dy1 = -dy1;
-    }
+    dy1 = -FRound(y2 - y1);
     ccw = (dx0 * dy1) >= (dx1 * dy0);
     return ccw;
 }
 
 static void
-DoHBendsNxt(Fixed x0, Fixed y0, Fixed x1, Fixed y1, PPathElt p)
+DoHBendsNxt(Fixed x0, Fixed y0, Fixed x1, Fixed y1, PathElt* p)
 {
-    Fixed x2, y2, delta, strt, end, x3, y3;
-    bool ysame, ccw, above, doboth;
+    Fixed x2, y2, x3, y3;
+    bool ysame;
     if (y0 == y1)
         return;
-    (void)NxtForBend(p, &x2, &y2, &x3, &y3);
+    NxtForBend(p, &x2, &y2, &x3, &y3);
     ysame = ProdLt0(y2 - y1, y1 - y0); /* y0 and y2 on same side of y1 */
     if (ysame ||
         (TestTan(x1 - x2, y1 - y2) &&
          (ProdLt0(x2 - x1, x1 - x0) ||
           (IsVertical(x0, y0, x1, y1) && TestBend(x0, y0, x1, y1, x2, y2))))) {
-        delta = FixHalfMul(gBendLength);
-        doboth = false;
+        Fixed strt, end;
+        Fixed delta = FixHalfMul(gBendLength);
+        bool doboth = false;
         if ((x0 <= x1 && x1 < x2) || (x0 < x1 && x1 <= x2)) {
+            /* do nothing */
         } else if ((x2 < x1 && x1 <= x0) || (x2 <= x1 && x1 < x0))
             delta = -delta;
         else if (ysame) {
-            above = y0 > y1;
-            if (!gYgoesUp)
-                above = !above;
+            bool ccw;
+            bool above = y0 < y1;
             ccw = IsCCW(x0, y0, x1, y1, x2, y2);
             if (above != ccw)
                 delta = -delta;
@@ -233,122 +230,124 @@ DoHBendsNxt(Fixed x0, Fixed y0, Fixed x1, Fixed y1, PPathElt p)
             doboth = true;
         strt = x1 - delta;
         end = x1 + delta;
-        AddHSegment(strt, end, y1, p, (PPathElt)NULL, sBEND, 0);
+        AddHSegment(strt, end, y1, p, NULL, sBEND, 0);
         if (doboth)
-            AddHSegment(end, strt, y1, p, (PPathElt)NULL, sBEND, 1);
+            AddHSegment(end, strt, y1, p, NULL, sBEND, 1);
     }
 }
 
 static void
-DoHBendsPrv(Fixed x0, Fixed y0, Fixed x1, Fixed y1, PPathElt p)
+DoHBendsPrv(Fixed x0, Fixed y0, Fixed x1, Fixed y1, PathElt* p)
 {
-    Fixed x2, y2, delta, strt, end;
-    bool ysame, ccw, above, doboth;
+    Fixed x2, y2;
+    bool ysame;
     if (y0 == y1)
         return;
-    (void)PrvForBend(p, &x2, &y2);
+    PrvForBend(p, &x2, &y2);
     ysame = ProdLt0(y2 - y0, y0 - y1);
     if (ysame ||
         (TestTan(x0 - x2, y0 - y2) &&
          (ProdLt0(x2 - x0, x0 - x1) ||
           (IsVertical(x0, y0, x1, y1) && TestBend(x2, y2, x0, y0, x1, y1))))) {
-        delta = FixHalfMul(gBendLength);
-        doboth = false;
+        Fixed strt, end;
+        Fixed delta = FixHalfMul(gBendLength);
+        bool doboth = false;
         if ((x2 < x0 && x0 <= x1) || (x2 <= x0 && x0 < x1)) {
+            /* do nothing */
         } else if ((x1 < x0 && x0 <= x2) || (x1 <= x0 && x0 < x2))
             delta = -delta;
         else if (ysame) {
-            above = (y2 > y0);
-            if (!gYgoesUp)
-                above = !above;
+            bool ccw;
+            bool above = y2 < y0;
             ccw = IsCCW(x2, y2, x0, y0, x1, y1);
             if (above != ccw)
                 delta = -delta;
         }
         strt = x0 - delta;
         end = x0 + delta;
-        AddHSegment(strt, end, y0, p->prev, (PPathElt)NULL, sBEND, 2);
+        AddHSegment(strt, end, y0, p->prev, NULL, sBEND, 2);
         if (doboth)
-            AddHSegment(end, strt, y0, p->prev, (PPathElt)NULL, sBEND, 3);
+            AddHSegment(end, strt, y0, p->prev, NULL, sBEND, 3);
     }
 }
 
 static void
-DoVBendsNxt(Fixed x0, Fixed y0, Fixed x1, Fixed y1, PPathElt p)
+DoVBendsNxt(Fixed x0, Fixed y0, Fixed x1, Fixed y1, PathElt* p)
 {
-    Fixed x2, y2, delta, strt, end, x3, y3;
-    bool xsame, ccw, right, doboth;
+    Fixed x2, y2, x3, y3;
+    bool xsame;
     if (x0 == x1)
         return;
-    (void)NxtForBend(p, &x2, &y2, &x3, &y3);
+    NxtForBend(p, &x2, &y2, &x3, &y3);
     xsame = ProdLt0(x2 - x1, x1 - x0);
     if (xsame ||
         (TestTan(y1 - y2, x1 - x2) &&
          (ProdLt0(y2 - y1, y1 - y0) || (IsHorizontal(x0, y0, x1, y1) &&
                                         TestBend(x0, y0, x1, y1, x2, y2))))) {
-        delta = FixHalfMul(gBendLength);
-        doboth = false;
+        Fixed strt, end;
+        Fixed delta = FixHalfMul(gBendLength);
+        bool doboth = false;
         if ((y0 <= y1 && y1 < y2) || (y0 < y1 && y1 <= y2)) {
+            /* do nothing */
         } else if ((y2 < y1 && y1 <= y0) || (y2 <= y1 && y1 < y0))
             delta = -delta;
         else if (xsame) {
-            right = x0 > x1;
-            ccw = IsCCW(x0, y0, x1, y1, x2, y2);
+            bool right = x0 > x1;
+            bool ccw = IsCCW(x0, y0, x1, y1, x2, y2);
             if (right != ccw)
                 delta = -delta;
-            if (!gYgoesUp)
-                delta = -delta;
+            delta = -delta;
         } else
             doboth = true;
         strt = y1 - delta;
         end = y1 + delta;
-        AddVSegment(strt, end, x1, p, (PPathElt)NULL, sBEND, 0);
+        AddVSegment(strt, end, x1, p, NULL, sBEND, 0);
         if (doboth)
-            AddVSegment(end, strt, x1, p, (PPathElt)NULL, sBEND, 1);
+            AddVSegment(end, strt, x1, p, NULL, sBEND, 1);
     }
 }
 
 static void
-DoVBendsPrv(Fixed x0, Fixed y0, Fixed x1, Fixed y1, PPathElt p)
+DoVBendsPrv(Fixed x0, Fixed y0, Fixed x1, Fixed y1, PathElt* p)
 {
-    Fixed x2, y2, delta, strt, end;
-    bool xsame, ccw, right, doboth;
+    Fixed x2, y2;
+    bool xsame;
     if (x0 == x1)
         return;
-    (void)PrvForBend(p, &x2, &y2);
+    PrvForBend(p, &x2, &y2);
     xsame = ProdLt0(x2 - x0, x0 - x1);
     if (xsame ||
         (TestTan(y0 - y2, x0 - x2) &&
          (ProdLt0(y2 - y0, y0 - y1) || (IsHorizontal(x0, y0, x1, y1) &&
                                         TestBend(x2, y2, x0, y0, x1, y1))))) {
-        delta = FixHalfMul(gBendLength);
-        doboth = false;
+        Fixed strt, end;
+        Fixed delta = FixHalfMul(gBendLength);
+        bool doboth = false;
         if ((y2 < y0 && y0 <= y1) || (y2 <= y0 && y0 < y1)) {
+            /* do nothing */
         } else if ((y1 < y0 && y0 <= y2) || (y1 <= y0 && y0 < y2))
             delta = -delta;
         else if (xsame) {
-            right = x0 > x1;
-            ccw = IsCCW(x2, y2, x0, y0, x1, y1);
+            bool right = x0 > x1;
+            bool ccw = IsCCW(x2, y2, x0, y0, x1, y1);
             if (right != ccw)
                 delta = -delta;
-            if (!gYgoesUp)
-                delta = -delta;
+            delta = -delta;
         }
         strt = y0 - delta;
         end = y0 + delta;
-        AddVSegment(strt, end, x0, p->prev, (PPathElt)NULL, sBEND, 2);
+        AddVSegment(strt, end, x0, p->prev, NULL, sBEND, 2);
         if (doboth)
-            AddVSegment(end, strt, x0, p->prev, (PPathElt)NULL, sBEND, 3);
+            AddVSegment(end, strt, x0, p->prev, NULL, sBEND, 3);
     }
 }
 
 static void
-MergeLnkSegs(PClrSeg seg1, PClrSeg seg2, PSegLnkLst lst)
+MergeLnkSegs(HintSeg* seg1, HintSeg* seg2, SegLnkLst* lst)
 {
     /* replace lnk refs to seg1 by seg2 */
-    PSegLnk lnk;
     while (lst != NULL) {
-        lnk = lst->lnk;
+        SegLnk* lnk = lst->lnk;
         if (lnk->seg == seg1)
             lnk->seg = seg2;
         lst = lst->next;
@@ -356,19 +355,19 @@ MergeLnkSegs(PClrSeg seg1, PClrSeg seg2, PSegLnkLst lst)
 }
 
 static void
-MergeHSegs(PClrSeg seg1, PClrSeg seg2)
+MergeHSegs(HintSeg* seg1, HintSeg* seg2)
 {
     MergeLnkSegs(seg1, seg2, Hlnks);
 }
 
 static void
-MergeVSegs(PClrSeg seg1, PClrSeg seg2)
+MergeVSegs(HintSeg* seg1, HintSeg* seg2)
 {
     MergeLnkSegs(seg1, seg2, Vlnks);
 }
 
 static void
-ReportRemSeg(int32_t l, PClrSeg lst)
+ReportRemSeg(int32_t l, HintSeg* lst)
 {
     Fixed from = 0, to = 0, loc = 0;
     /* this assumes !YgoesUp */
@@ -388,11 +387,13 @@ ReportRemSeg(int32_t l, PClrSeg lst)
     switch (l) {
         case 0:
         case 1:
-            ReportRemVSeg(from, to, loc);
+            LogMsg(LOGDEBUG, OK, "rem vseg %g %g to %g %g", FixToDbl(loc),
+                   FixToDbl(-from), FixToDbl(loc), FixToDbl(-to));
             break;
         case 2:
         case 3:
-            ReportRemHSeg(from, to, loc);
+            LogMsg(LOGDEBUG, OK, "rem hseg %g %g to %g %g", FixToDbl(from),
+                   FixToDbl(-loc), FixToDbl(to), FixToDbl(-loc));
             break;
     }
 }
@@ -401,19 +402,16 @@ ReportRemSeg(int32_t l, PClrSeg lst)
 static void
 RemExtraBends(int32_t l0, int32_t l1)
 {
-    PClrSeg lst0, lst, n, p;
-    PClrSeg nxt, prv;
-    Fixed loc0, loc;
-    lst0 = gSegLists[l0];
-    prv = NULL;
+    HintSeg* lst0 = gSegLists[l0];
+    HintSeg* prv = NULL;
     while (lst0 != NULL) {
-        nxt = lst0->sNxt;
-        loc0 = lst0->sLoc;
-        lst = gSegLists[l1];
-        p = NULL;
+        HintSeg* nxt = lst0->sNxt;
+        Fixed loc0 = lst0->sLoc;
+        HintSeg* lst = gSegLists[l1];
+        HintSeg* p = NULL;
         while (lst != NULL) {
-            n = lst->sNxt;
-            loc = lst->sLoc;
+            HintSeg* n = lst->sNxt;
+            Fixed loc = lst->sLoc;
             if (loc > loc0)
                 break; /* list in increasing order by sLoc */
             if (loc == loc0 && lst->sMin < lst0->sMax &&
@@ -426,8 +424,7 @@ RemExtraBends(int32_t l0, int32_t l1)
                         gSegLists[l0] = nxt;
                     else
                         prv->sNxt = nxt;
-                    if (gDebug)
-                        ReportRemSeg(l0, lst0);
+                    ReportRemSeg(l0, lst0);
                     lst0 = prv;
                     break;
                 }
@@ -439,8 +436,7 @@ RemExtraBends(int32_t l0, int32_t l1)
                         gSegLists[l1] = n;
                     else
                         p->sNxt = n;
-                    if (gDebug)
-                        ReportRemSeg(l1, lst);
+                    ReportRemSeg(l1, lst);
                     lst = p;
                 }
             }
@@ -453,17 +449,16 @@ RemExtraBends(int32_t l0, int32_t l1)
 }
 
 static void
-CompactList(int32_t i, void (*nm)(PClrSeg, PClrSeg))
+CompactList(int32_t i, void (*nm)(HintSeg*, HintSeg*))
 {
-    PClrSeg lst, prv, nxtprv, nxt;
-    Fixed lstmin, lstmax, nxtmin, nxtmax;
-    bool flg;
-    lst = gSegLists[i];
-    prv = NULL;
+    HintSeg* lst = gSegLists[i];
+    HintSeg* prv = NULL;
     while (lst != NULL) {
-        nxt = lst->sNxt;
-        nxtprv = lst;
+        bool flg;
+        HintSeg* nxt = lst->sNxt;
+        HintSeg* nxtprv = lst;
         while (true) {
+            Fixed lstmin, lstmax, nxtmin, nxtmax;
             if ((nxt == NULL) || (nxt->sLoc > lst->sLoc)) {
                 flg = true;
                 break;
@@ -474,7 +469,7 @@ CompactList(int32_t i, void (*nm)(PClrSeg, PClrSeg))
             nxtmax = nxt->sMax;
             if (lstmax >= nxtmin && lstmin <= nxtmax) {
                 /* do not worry about YgoesUp since "sMax" is really max in
-                 device space, not in character space */
+                 device space, not in glyph space */
                 if (abs(lstmax - lstmin) > abs(nxtmax - nxtmin)) {
                     /* merge into lst and remove nxt */
                     (*nm)(nxt, lst);
@@ -564,34 +559,34 @@ TstFlat(Fixed dmn, Fixed dmx)
 }
 
 static bool
-NxtHorz(Fixed x, Fixed y, PPathElt p)
+NxtHorz(Fixed x, Fixed y, PathElt* p)
 {
     Fixed x2, y2, x3, y3;
-    p = NxtForBend(p, &x2, &y2, &x3, &y3);
+    NxtForBend(p, &x2, &y2, &x3, &y3);
     return TstFlat(y2 - y, x2 - x);
 }
 
 static bool
-PrvHorz(Fixed x, Fixed y, PPathElt p)
+PrvHorz(Fixed x, Fixed y, PathElt* p)
 {
     Fixed x2, y2;
-    p = PrvForBend(p, &x2, &y2);
+    PrvForBend(p, &x2, &y2);
     return TstFlat(y2 - y, x2 - x);
 }
 
 static bool
-NxtVert(Fixed x, Fixed y, PPathElt p)
+NxtVert(Fixed x, Fixed y, PathElt* p)
 {
     Fixed x2, y2, x3, y3;
-    p = NxtForBend(p, &x2, &y2, &x3, &y3);
+    NxtForBend(p, &x2, &y2, &x3, &y3);
     return TstFlat(x2 - x, y2 - y);
 }
 
 static bool
-PrvVert(Fixed x, Fixed y, PPathElt p)
+PrvVert(Fixed x, Fixed y, PathElt* p)
 {
     Fixed x2, y2;
-    p = PrvForBend(p, &x2, &y2);
+    PrvForBend(p, &x2, &y2);
     return TstFlat(x2 - x, y2 - y);
 }
 
@@ -607,7 +602,7 @@ TstSameDir(Fixed x0, Fixed y0, Fixed x1, Fixed y1, Fixed x2, Fixed y2)
 }
 
 static bool
-PrvSameDir(Fixed x0, Fixed y0, Fixed x1, Fixed y1, PPathElt p)
+PrvSameDir(Fixed x0, Fixed y0, Fixed x1, Fixed y1, PathElt* p)
 {
     Fixed x2, y2;
     p = PrvForBend(p, &x2, &y2);
@@ -617,7 +612,7 @@ PrvSameDir(Fixed x0, Fixed y0, Fixed x1, Fixed y1, PPathElt p)
 }
 
 static bool
-NxtSameDir(Fixed x0, Fixed y0, Fixed x1, Fixed y1, PPathElt p)
+NxtSameDir(Fixed x0, Fixed y0, Fixed x1, Fixed y1, PathElt* p)
 {
     Fixed x2, y2, x3, y3;
     p = NxtForBend(p, &x2, &y2, &x3, &y3);
@@ -629,10 +624,10 @@ NxtSameDir(Fixed x0, Fixed y0, Fixed x1, Fixed y1, PPathElt p)
 }
 
 void
-GenVPts(int32_t specialCharType)
+GenVPts(int32_t specialGlyphType)
 {
-    /* specialCharType 1 = upper; -1 = lower; 0 = neither */
-    PPathElt p, fl;
+    /* specialGlyphType 1 = upper; -1 = lower; 0 = neither */
+    PathElt *p, *fl;
     bool isVert, flex1, flex2;
     Fixed flx0, fly0, llx, lly, urx, ury, yavg, yend, ydist, q, q2;
     Fixed prvx, prvy, nxtx, nxty, xx, yy, yd2;
@@ -709,12 +704,12 @@ GenVPts(int32_t specialCharType)
                             ReportLinearCurve(p, x0, y0, x1, y1);
                         ydist = FixHalfMul(yd2);
                         yavg = FixHalfMul(y0 + y1);
-                        (void)PrvForBend(p, &prvx, &prvy);
-                        (void)NxtForBend(p, &nxtx, &nxty, &xx, &yy);
+                        PrvForBend(p, &prvx, &prvy);
+                        NxtForBend(p, &nxtx, &nxty, &xx, &yy);
                         AddVSegment(yavg - ydist, yavg + ydist,
                                     PickVSpot(x0, y0, x1, y1, px1, py1, px2,
                                               py2, prvx, prvy, nxtx, nxty),
-                                    p, (PPathElt)NULL, sCURVE, 7);
+                                    p, NULL, sCURVE, 7);
                     } else {
                         q2 = VertQuo(px1, py1, x1, y1);
                         if (q2 > 0 && ProdGe0(py1 - y1, py2 - y1) &&
@@ -722,11 +717,9 @@ GenVPts(int32_t specialCharType)
                             yend = AdjDist(y1 - CPFrom(py1, py2), q2);
                             if (abs(yend) > abs(ydist))
                                 ydist = yend;
-                            AddVSegment(y1 - ydist, y1, x1, p, (PPathElt)NULL,
-                                        sCURVE, 8);
+                            AddVSegment(y1 - ydist, y1, x1, p, NULL, sCURVE, 8);
                         } else
-                            AddVSegment(y1 - ydist, y1, x1, p, (PPathElt)NULL,
-                                        sCURVE, 9);
+                            AddVSegment(y1 - ydist, y1, x1, p, NULL, sCURVE, 9);
                     }
                 }
             }
@@ -748,17 +741,17 @@ GenVPts(int32_t specialCharType)
                         if (abs(ydist) < gBendLength)
                             ydist = (ydist > 0) ? FixHalfMul(gBendLength)
                                                 : FixHalfMul(-gBendLength);
-                        AddVSegment(yavg - ydist, yavg + ydist, loc, p,
-                                    (PPathElt)NULL, sCURVE, 10);
+                        AddVSegment(yavg - ydist, yavg + ydist, loc, p, NULL,
+                                    sCURVE, 10);
                     }
                 }
             }
         } else if (p->type == MOVETO) {
             gBonus = 0;
-            if (specialCharType == -1) {
+            if (specialGlyphType == -1) {
                 if (IsLower(p))
                     gBonus = FixInt(200);
-            } else if (specialCharType == 1) {
+            } else if (specialGlyphType == 1) {
                 if (IsUpper(p))
                     gBonus = FixInt(200);
             }
@@ -771,12 +764,12 @@ GenVPts(int32_t specialCharType)
                         q = FixQuarter;
                     ydist = FixHalfMul(AdjDist(y1 - y0, q));
                     yavg = FixHalfMul(y0 + y1);
-                    (void)PrvForBend(p, &prvx, &prvy);
-                    (void)NxtForBend(p, &nxtx, &nxty, &xx, &yy);
+                    PrvForBend(p, &prvx, &prvy);
+                    NxtForBend(p, &nxtx, &nxty, &xx, &yy);
                     AddVSegment(yavg - ydist, yavg + ydist,
                                 PickVSpot(x0, y0, x1, y1, x0, y0, x1, y1, prvx,
                                           prvy, nxtx, nxty),
-                                p, (PPathElt)NULL, sLINE, 12);
+                                p, NULL, sLINE, 12);
                     if (abs(x0 - x1) <= FixTwo)
                         ReportNonVError(x0, y0, x1, y1);
                 }
@@ -801,10 +794,10 @@ InBlueBand(Fixed loc, int32_t n, Fixed* p)
     Fixed y;
     if (n <= 0)
         return false;
-    y = itfmy(loc);
+    y = -loc;
     /* Augment the blue band by bluefuzz in each direction.  This will
- result in "near misses" being colored and so adjusted by the
- PS interpreter. */
+       result in "near misses" being hinted and so adjusted by the
+       PS interpreter. */
     for (i = 0; i < n; i += 2)
         if ((p[i] - gBlueFuzz) <= y && (p[i + 1] + gBlueFuzz) >= y)
             return true;
@@ -817,7 +810,6 @@ PickHSpot(Fixed x0, Fixed y0, Fixed x1, Fixed y1, Fixed xdist, Fixed px1,
           Fixed nxty)
 {
     bool topSeg = (xdist < 0) ? true : false;
-    Fixed upper, lower;
     bool inBlue0, inBlue1;
     if (topSeg) {
         inBlue0 = InBlueBand(y0, gLenTopBands, gTopBands);
@@ -839,17 +831,13 @@ PickHSpot(Fixed x0, Fixed y0, Fixed x1, Fixed y1, Fixed xdist, Fixed px1,
     if (y0 != prvy && y1 == nxty)
         return y1;
     if (inBlue0 && inBlue1) {
+        Fixed upper, lower;
         if (y0 > y1) {
-            upper = y0;
-            lower = y1;
-        } else {
             upper = y1;
             lower = y0;
-        }
-        if (!gYgoesUp) {
-            Fixed tmp = lower;
-            lower = upper;
-            upper = tmp;
+        } else {
+            upper = y0;
+            lower = y1;
         }
         return topSeg ? upper : lower;
     }
@@ -868,7 +856,7 @@ PickHSpot(Fixed x0, Fixed y0, Fixed x1, Fixed y1, Fixed xdist, Fixed px1,
 void
 GenHPts(void)
 {
-    PPathElt p, fl;
+    PathElt *p, *fl;
     bool isHoriz, flex1, flex2;
     Fixed flx0, fly0, llx, lly, urx, ury, xavg, xend, xdist, q, q2;
     Fixed prvx, prvy, nxtx, nxty, xx, yy, xd2;
@@ -943,14 +931,14 @@ GenHPts(void)
                         Fixed hspot;
                         if (y0 == py1 && py1 == py2 && py2 == y1)
                             ReportLinearCurve(p, x0, y0, x1, y1);
-                        (void)PrvForBend(p, &prvx, &prvy);
-                        (void)NxtForBend(p, &nxtx, &nxty, &xx, &yy);
+                        PrvForBend(p, &prvx, &prvy);
+                        NxtForBend(p, &nxtx, &nxty, &xx, &yy);
                         xdist = FixHalfMul(xd2);
                         xavg = FixHalfMul(x0 + x1);
                         hspot = PickHSpot(x0, y0, x1, y1, xdist, px1, py1, px2,
                                           py2, prvx, prvy, nxtx, nxty);
-                        AddHSegment(xavg - xdist, xavg + xdist, hspot, p,
-                                    (PPathElt)NULL, sCURVE, 7);
+                        AddHSegment(xavg - xdist, xavg + xdist, hspot, p, NULL,
+                                    sCURVE, 7);
                     } else {
                         q2 = HorzQuo(px1, py1, x1, y1);
                         if (q2 > 0 && ProdGe0(px1 - x1, px2 - x1) &&
@@ -958,11 +946,9 @@ GenHPts(void)
                             xend = AdjDist(x1 - CPFrom(px1, px2), q2);
                             if (abs(xend) > abs(xdist))
                                 xdist = xend;
-                            AddHSegment(x1 - xdist, x1, y1, p, (PPathElt)NULL,
-                                        sCURVE, 8);
+                            AddHSegment(x1 - xdist, x1, y1, p, NULL, sCURVE, 8);
                         } else
-                            AddHSegment(x1 - xdist, x1, y1, p, (PPathElt)NULL,
-                                        sCURVE, 9);
+                            AddHSegment(x1 - xdist, x1, y1, p, NULL, sCURVE, 9);
                     }
                 }
             }
@@ -984,8 +970,8 @@ GenHPts(void)
                         if (abs(xdist) < gBendLength)
                             xdist = (xdist > 0.0) ? FixHalfMul(gBendLength)
                                                   : FixHalfMul(-gBendLength);
-                        AddHSegment(xavg - xdist, xavg + xdist, loc, p,
-                                    (PPathElt)NULL, sCURVE, 10);
+                        AddHSegment(xavg - xdist, xavg + xdist, loc, p, NULL,
+                                    sCURVE, 10);
                     }
                 }
             }
@@ -998,8 +984,8 @@ GenHPts(void)
                         q = FixQuarter;
                     xdist = FixHalfMul(AdjDist(x1 - x0, q));
                     xavg = FixHalfMul(x0 + x1);
-                    (void)PrvForBend(p, &prvx, &prvy);
-                    (void)NxtForBend(p, &nxtx, &nxty, &xx, &yy);
+                    PrvForBend(p, &prvx, &prvy);
+                    NxtForBend(p, &nxtx, &nxty, &xx, &yy);
                     yy = PickHSpot(x0, y0, x1, y1, xdist, x0, y0, x1, y1, prvx,
                                    prvy, nxtx, nxty);
                     AddHSegment(xavg - xdist, xavg + xdist, yy, p->prev, p,
