@@ -18,30 +18,6 @@ ACBuffer* gBezOutput = NULL;
 
 static jmp_buf aclibmark; /* to handle exit() calls in the library version*/
 
-static ACBuffer*
-NewBuffer(size_t size)
-{
-    ACBuffer* buffer;
-
-    buffer = (ACBuffer*)AllocateMem(1, sizeof(ACBuffer), "out buffer");
-    buffer->data = AllocateMem(size, 1, "out buffer data");
-    buffer->data[0] = '\0';
-    buffer->capacity = size;
-    buffer->length = 0;
-
-    return buffer;
-}
-
-static void
-FreeBuffer(ACBuffer* buffer)
-{
-    if (!buffer)
-        return;
-
-    UnallocateMem(buffer->data);
-    UnallocateMem(buffer);
-}
-
 ACLIB_API void
 AC_SetMemManager(void* ctxptr, AC_MEMMANAGEFUNCPTR func)
 {
@@ -56,11 +32,12 @@ AC_SetReportCB(AC_REPORTFUNCPTR reportCB)
 
 ACLIB_API void
 AC_SetReportStemsCB(AC_REPORTSTEMPTR hstemCB, AC_REPORTSTEMPTR vstemCB,
-                    unsigned int allStems)
+                    unsigned int allStems, void* userData)
 {
     gAllStems = allStems;
     gAddHStemCB = hstemCB;
     gAddVStemCB = vstemCB;
+    gAddStemUserData = userData;
     gDoStems = true;
 
     gAddGlyphExtremesCB = NULL;
@@ -69,10 +46,12 @@ AC_SetReportStemsCB(AC_REPORTSTEMPTR hstemCB, AC_REPORTSTEMPTR vstemCB,
 }
 
 ACLIB_API void
-AC_SetReportZonesCB(AC_REPORTZONEPTR glyphCB, AC_REPORTZONEPTR stemCB)
+AC_SetReportZonesCB(AC_REPORTZONEPTR glyphCB, AC_REPORTZONEPTR stemCB,
+                    void* userData)
 {
     gAddGlyphExtremesCB = glyphCB;
     gAddStemExtremesCB = stemCB;
+    gAddExtremesUserData = userData;
     gDoAligns = true;
 
     gAddHStemCB = NULL;
@@ -81,9 +60,10 @@ AC_SetReportZonesCB(AC_REPORTZONEPTR glyphCB, AC_REPORTZONEPTR stemCB)
 }
 
 ACLIB_API void
-AC_SetReportRetryCB(AC_RETRYPTR retryCB)
+AC_SetReportRetryCB(AC_RETRYPTR retryCB, void* userData)
 {
     gReportRetryCB = retryCB;
+    gReportRetryUserData = userData;
 }
 
 /*
@@ -106,93 +86,12 @@ error_handler(int16_t code)
 
 ACLIB_API int
 AutoHintString(const char* srcbezdata, const char* fontinfodata,
-               char** dstbezdata, size_t* length, int allowEdit,
-               int allowHintSub, int roundCoords)
+               ACBuffer* outbuffer, int allowEdit, int allowHintSub,
+               int roundCoords)
 {
     int value, result;
     ACFontInfo* fontinfo = NULL;
 
-    if (!srcbezdata || !*length)
-        return AC_InvalidParameterError;
-
-    fontinfo = ParseFontInfo(fontinfodata);
-
-    set_errorproc(error_handler);
-    value = setjmp(aclibmark);
-
-    /* We will return here whenever an error occurs during the execution of
-     * AutoHint(), or after it finishes execution. See the error_handler
-     * comments above and below. */
-
-    if (value == -1) {
-        /* a fatal error occurred somewhere. */
-        FreeFontInfo(fontinfo);
-        return AC_FatalError;
-    } else if (value == 1) {
-        /* AutoHint was called successfully */
-        FreeFontInfo(fontinfo);
-
-        if (gBezOutput->length >= *length)
-            *dstbezdata = ReallocateMem(*dstbezdata, gBezOutput->length + 1,
-                                        "Output buffer");
-
-        *length = gBezOutput->length + 1;
-        strncpy(*dstbezdata, gBezOutput->data, *length);
-
-        FreeBuffer(gBezOutput);
-
-        return AC_Success;
-    }
-
-    gBezOutput = NewBuffer(*length);
-    result = AutoHint(fontinfo,     /* font info */
-                      srcbezdata,   /* input glyph */
-                      allowHintSub, /* extrahint */
-                      allowEdit,    /* changeGlyphs */
-                      roundCoords);
-    /* result == true is good */
-
-    /* The following call to error_handler() always returns control to just
-     * after the setjmp() function call above, but with value set to 1 if
-     * success, or -1 if not */
-    error_handler((result == true) ? OK : NONFATALERROR);
-
-    /* Shouldn't get here */
-    return AC_UnknownError;
-}
-
-ACLIB_API int
-AutoHintStringMM(const char** srcbezdata, const char* fontinfodata,
-                 int nmasters, const char** masters, char** dstbezdata,
-                 size_t* lengths)
-{
-    /* Only the master with index 'hintsMasterIx' needs to be hinted; this is
-     * why only the fontinfo data for that master is needed. This function
-     * expects that the master with index 'hintsMasterIx' has already been
-     * hinted with AutoHint().
-     *
-     * The hints for the others masters are derived a very simple process. When
-     * the first master was hinted, the logic recorded the path element index
-     * for the path element which sets the inner or outer edge of each hint,
-     * and whether it is the endpoint or startpoint which sets the edge. For
-     * each other master, the logic gets the path element for the current
-     * master using the same path element index as in the first master, and
-     * uses the end or start point of that path element to set the edge in the
-     * current master.
-     *
-     * Some code notes: The original hinting pass in AutoHintString() on the
-     * first master records the path indicies for each path element that sets a
-     * hint edge, and whether it is a start or end point; this stored in the
-     * hintElt structures in the first master. There is a hintElt for the main
-     * (default) hints at the start of the charstring, and there is hintElt
-     * structure attached to each path element which triggers a new hint set.
-     * The function charpath.c::ReadandAssignHints(), called from
-     * charpath.c::MergeGlyphPaths(), then copies all the hintElts to the
-     * current master main or path elements. (This actually happens in
-     * charpath.c::InsertHint().) */
-    int value, result;
-    ACFontInfo* fontinfo = NULL;
-
     if (!srcbezdata)
         return AC_InvalidParameterError;
 
@@ -212,12 +111,76 @@ AutoHintStringMM(const char** srcbezdata, const char* fontinfodata,
     } else if (value == 1) {
         /* AutoHint was called successfully */
         FreeFontInfo(fontinfo);
+
+        return AC_Success;
+    }
+
+    gBezOutput = outbuffer;
+    result = AutoHint(fontinfo,     /* font info */
+                      srcbezdata,   /* input glyph */
+                      allowHintSub, /* extrahint */
+                      allowEdit,    /* changeGlyphs */
+                      roundCoords);
+    /* result == true is good */
+
+    /* The following call to error_handler() always returns control to just
+     * after the setjmp() function call above, but with value set to 1 if
+     * success, or -1 if not */
+    error_handler((result == true) ? OK : NONFATALERROR);
+
+    /* Shouldn't get here */
+    return AC_UnknownError;
+}
+
+ACLIB_API int
+AutoHintStringMM(const char** srcbezdata, int nmasters, const char** masters,
+                 ACBuffer** outbuffers)
+{
+    /* Only the master with index 'hintsMasterIx' needs to be hinted.
+     * This function expects that the master with index 'hintsMasterIx' has
+     * already been hinted with AutoHint().
+     *
+     * The hints for the others masters are derived a very simple process. When
+     * the first master was hinted, the logic recorded the path element index
+     * for the path element which sets the inner or outer edge of each hint,
+     * and whether it is the endpoint or startpoint which sets the edge. For
+     * each other master, the logic gets the path element for the current
+     * master using the same path element index as in the first master, and
+     * uses the end or start point of that path element to set the edge in the
+     * current master.
+     *
+     * Some code notes: The original hinting pass in AutoHintString() on the
+     * first master records the path indicies for each path element that sets a
+     * hint edge, and whether it is a start or end point; this stored in the
+     * hintElt structures in the first master. There is a hintElt for the main
+     * (default) hints at the start of the charstring, and there is hintElt
+     * structure attached to each path element which triggers a new hint set.
+     * The function charpath.c::ReadandAssignHints(), called from
+     * charpath.c::MergeGlyphPaths(), then copies all the hintElts to the
+     * current master main or path elements. (This actually happens in
+     * charpath.c::InsertHint().) */
+    int value, result;
+
+    if (!srcbezdata)
+        return AC_InvalidParameterError;
+
+    set_errorproc(error_handler);
+    value = setjmp(aclibmark);
+
+    /* We will return here whenever an error occurs during the execution of
+     * AutoHint(), or after it finishes execution. See the error_handler
+     * comments above and below. */
+
+    if (value == -1) {
+        /* a fatal error occurred somewhere. */
+        return AC_FatalError;
+    } else if (value == 1) {
+        /* AutoHint was called successfully */
         return AC_Success;
     }
 
     /* result == true is good */
-    result = MergeGlyphPaths(fontinfo, srcbezdata, nmasters, masters,
-                             dstbezdata, lengths);
+    result = MergeGlyphPaths(srcbezdata, nmasters, masters, outbuffers);
 
     /* The following call to error_handler() always returns control to just
      * after the setjmp() function call above, but with value set to 1 if
@@ -238,6 +201,10 @@ AC_initCallGlobals(void)
     gAddHStemCB = NULL;
     gAddVStemCB = NULL;
     gDoStems = false;
+    gAddStemUserData = NULL;
+    gAllStems = 0;
+    gReportRetryCB = NULL;
+    gReportRetryUserData = NULL;
 }
 
 ACLIB_API const char*
