@@ -4,16 +4,13 @@
 Auto-hinting program for PostScript, OpenType/CFF and UFO fonts.
 """
 
-from __future__ import print_function, absolute_import
-
 import argparse
 import logging
 import os
 import re
 import subprocess
 import sys
-
-from fontTools.misc.py23 import open
+import textwrap
 
 from . import __version__, get_font_format
 from .autohint import ACOptions, hintFiles
@@ -39,7 +36,7 @@ The reports provided by the stemHist tool can be useful for choosing alignment
 zone and stem width values.
 """
 
-FDDICT_DOC = """
+FDDICT_DOC = r"""
 By default, psautohint uses the font's global alignment zones and stem widths
 when hinting each glyph. However, if there is a file named 'fontinfo' in the
 same directory as the input font file, psautohint will search it for
@@ -321,9 +318,9 @@ def _process_glyph_list_arg(glyph_list, name_aliases):
     return [_expand_cid_name(n, name_aliases) for n in glyph_list]
 
 
-class Options(ACOptions):
+class HintOptions(ACOptions):
     def __init__(self, pargs):
-        super(Options, self).__init__()
+        super(HintOptions, self).__init__()
         self.inputPaths = pargs.font_paths
         self.outputPaths = pargs.output_paths
         self.reference_font = pargs.reference_font
@@ -343,16 +340,33 @@ class _CustomHelpFormatter(argparse.RawDescriptionHelpFormatter):
     """
     Adds extra line between options
     """
+    @staticmethod
+    def __add_whitespace(i, i_wtsp, arg):
+        if i == 0:
+            return arg
+        return (" " * i_wtsp) + arg
+
     def _split_lines(self, arg, width):
+        arg_rows = arg.splitlines()
+        for i, line in enumerate(arg_rows):
+            search = re.search(r'\s*[0-9\-]{0,}\.?\s*', line)
+            if line.strip() == "":
+                arg_rows[i] = " "
+            elif search:
+                line_wtsp = search.end()
+                lines = [self.__add_whitespace(j, line_wtsp, x)
+                         for j, x in enumerate(textwrap.wrap(line, width))]
+                arg_rows[i] = lines
+
         # [''] adds the extra line between args
-        return super(_CustomHelpFormatter,
-                     self)._split_lines(arg, width) + ['']
+        return [item for sublist in arg_rows for item in sublist] + ['']
 
 
 class _AdditionalHelpAction(argparse.Action):
     """
     Based on argparse's _HelpAction and _VersionAction.
     """
+
     def __init__(self,
                  option_strings,
                  addl_help=None,
@@ -373,6 +387,26 @@ class _AdditionalHelpAction(argparse.Action):
         parser.exit(message=formatter.format_help())
 
 
+class DuplicateMessageFilter(logging.Filter):
+    """
+    Suppresses any log message that was reported before in the same module and
+    for the same logging level. We check for module and level number in
+    addition to the message just in case, though checking the message only is
+    probably enough.
+    """
+
+    def __init__(self):
+        super(DuplicateMessageFilter, self).__init__()
+        self.logs = set()
+
+    def filter(self, record):
+        current = (record.module, record.levelno, record.getMessage())
+        if current in self.logs:
+            return False
+        self.logs.add(current)
+        return True
+
+
 def _split_comma_sequence(comma_str):
     return [item.strip() for item in comma_str.split(',')]
 
@@ -391,7 +425,7 @@ def _check_save_path(path_str):
             os.remove(check_path)
     except (IOError, OSError):
         raise argparse.ArgumentTypeError(
-            "{} is not a valid path to write to.".format(test_path))
+            f"{test_path} is not a valid path to write to.")
     return test_path
 
 
@@ -412,16 +446,15 @@ def _validate_font_paths(path_lst, parser):
     format_set = set()
     for path in path_lst:
         font_format = get_font_format(path)
+        base_name = os.path.basename(path)
         if font_format is None:
-            parser.error("{} is not a supported font format".format(
-                os.path.basename(path)))
+            parser.error(f"{base_name} is not a supported font format")
         if font_format in ("PFA", "PFB", "PFC"):
             if has_tx is None:
                 has_tx = _check_tx()
             if not has_tx:
-                parser.error("{} font format requires 'tx'. "
-                             "Please install 'afdko'.".format(
-                                 os.path.basename(path)))
+                parser.error(f"{base_name} font format requires 'tx'. "
+                             "Please install 'afdko'.")
         format_set.add(font_format)
     if len(format_set) > 1:
         parser.error("the input fonts must be all of the same format")
@@ -434,7 +467,7 @@ def _validate_path(path_str):
     valid_path = os.path.abspath(os.path.realpath(path_str))
     if not os.path.exists(valid_path):
         raise argparse.ArgumentTypeError(
-            "{} is not a valid path.".format(path_str))
+            f"{path_str} is not a valid path.")
     return valid_path
 
 
@@ -465,10 +498,10 @@ def get_options(args):
         formatter_class=_CustomHelpFormatter,
         description=__doc__
     )
-    parser.add_argument(
+    parser_fonts = parser.add_argument(
         'font_paths',
         metavar='FONT',
-        nargs='+',
+        nargs='*',
         type=_validate_path,
         help='Type1/CFF/OTF/UFO font files'
     )
@@ -606,7 +639,7 @@ def get_options(args):
         metavar='PATH',
         type=_validate_path,
         help='file containing hinting parameters\n'
-             "Default: '{}'".format(FONTINFO_FILE_NAME)
+             f"Default: '{FONTINFO_FILE_NAME}'"
     )
     parser.add_argument(
         '--print-dflt-fddict',
@@ -641,6 +674,12 @@ def get_options(args):
         action='version',
         version=__version__
     )
+    parser.add_argument(
+        '--traceback',
+        action='store_true',
+        help="show traceback for exceptions.",
+    )
+
     parsed_args = parser.parse_args(args)
 
     if parsed_args.verbose == 0:
@@ -652,6 +691,20 @@ def get_options(args):
 
     logging.basicConfig(format="%(levelname)s: %(message)s", level=log_level,
                         filename=parsed_args.log_path)
+    for handler in logging.root.handlers:
+        handler.addFilter(DuplicateMessageFilter())
+
+    if (not len(parsed_args.font_paths or []) and
+            len(parsed_args.output_paths or [])):
+        # allow "psautohint -o outputpath inputpath"
+        # see https://github.com/adobe-type-tools/psautohint/issues/129
+        half = len(parsed_args.output_paths) // 2
+        parsed_args.font_paths = [
+            parsed_args.output_paths.pop(half) for _ in range(half)]
+
+    if not len(parsed_args.font_paths):
+        parser.error(
+            f"the following arguments are required: {parser_fonts.metavar}")
 
     if (parsed_args.output_paths and
             len(parsed_args.font_paths) != len(parsed_args.output_paths)):
@@ -668,7 +721,7 @@ def get_options(args):
     else:
         all_font_paths = parsed_args.font_paths
 
-    options = Options(parsed_args)
+    options = HintOptions(parsed_args)
 
     options.font_format = _validate_font_paths(all_font_paths, parser)
 
@@ -697,17 +750,195 @@ def get_options(args):
     else:
         _parse_fontinfo_file(options, parsed_args.fontinfo_file)
 
-    return options
+    return options, parsed_args
 
 
 def main(args=None):
-    options = get_options(args)
+    options, pargs = get_options(args)
 
     try:
         hintFiles(options)
-    except Exception:
-        logging.exception("Unhandled exception occurred")
-        raise
+    except Exception as ex:
+        if pargs.traceback:
+            raise
+        logging.error(ex)
+        return 1
+
+
+class ReportOptions(ACOptions):
+    def __init__(self, pargs):
+        super(ReportOptions, self).__init__()
+        self.hintAll = True
+        self.noFlex = True
+        self.allow_no_blues = True
+        self.logOnly = True
+        self.inputPaths = pargs.font_paths
+        self.outputPaths = pargs.output_paths
+        self.report_stems = not pargs.alignment_zones
+        self.report_zones = pargs.alignment_zones
+        self.report_all_stems = pargs.all_stems
+
+
+def get_stemhist_options(args):
+    parser = argparse.ArgumentParser(
+        formatter_class=_CustomHelpFormatter,
+        description='Stem and Alignment zones report for PostScript, '
+                    'OpenType/CFF and UFO fonts.'
+    )
+    parser.add_argument(
+        'font_paths',
+        metavar='FONT',
+        nargs='+',
+        type=_validate_path,
+        help='Type1/CFF/OTF/UFO font files'
+    )
+    parser.add_argument(
+        '-o',
+        '--output',
+        metavar='PATH',
+        nargs='+',
+        dest='output_paths',
+        type=_check_save_path,
+        help='When this is specified, the path argument is used as the base '
+             'path for the reports. Otherwise, the font file path is used as '
+             'the base path. Several reports are written, using the base path '
+             'name plus an extension.\n'
+             'Without the -z option, the report files are:\n'
+             '   <base path>.hstm.txt  The horizontal stems.\n'
+             '   <base path>.vstm.txt  The vertical stems.\n'
+             'With the -z option, the report files are:\n'
+             '   <base path>.top.txt   The top zones.\n'
+             '   <base path>.bot.txt   The bottom zones.'
+    )
+    parser.add_argument(
+        '-z',
+        '--alignment-zones',
+        action='store_true',
+        dest='alignment_zones',
+        help='Print alignment zone report rather than stem report.'
+    )
+    parser.add_argument(
+        '-a',
+        '--all',
+        action='store_true',
+        dest='all_stems',
+        help='Include stems formed by curved line segments; by default, '
+             'includes only stems formed by straight line segments.'
+    )
+    glyphs_parser = parser.add_mutually_exclusive_group()
+    glyphs_parser.add_argument(
+        '-g',
+        '--glyphs',
+        metavar='GLYPH_LIST',
+        dest='glyphs_to_hint',
+        type=_split_comma_sequence,
+        default=[],
+        help='comma-separated sequence of glyphs to process\n'
+             'The glyph identifiers may be glyph indexes, glyph names, or '
+             'glyph CIDs. CID values must be prefixed with a forward slash.\n'
+             'Examples:\n'
+             '    psautohint -g A,B,C,69 MyFont.ufo\n'
+             '    psautohint -g /103,/434,68 MyCIDFont'
+    )
+    glyphs_parser.add_argument(
+        '--glyphs-file',
+        metavar='PATH',
+        dest='glyphs_to_hint_file',
+        type=_validate_path,
+        help='file containing a list of glyphs to process\n'
+             'The file must contain a comma-separated sequence of glyph '
+             'identifiers.'
+    )
+    glyphs_parser.add_argument(
+        '-x',
+        '--exclude-glyphs',
+        metavar='GLYPH_LIST',
+        dest='glyphs_to_not_hint',
+        type=_split_comma_sequence,
+        default=[],
+        help='comma-separated sequence of glyphs to NOT process\n'
+             "Counterpart to '--glyphs' option."
+    )
+    glyphs_parser.add_argument(
+        '--exclude-glyphs-file',
+        metavar='PATH',
+        dest='glyphs_to_not_hint_file',
+        type=_validate_path,
+        help='file containing a list of glyphs to NOT process\n'
+             "Counterpart to '--glyphs-file' option."
+    )
+    parser.add_argument(
+        '-v',
+        '--verbose',
+        action='count',
+        default=0,
+        help='verbose mode\n'
+             'Use -vv for extra-verbose mode.'
+    )
+    parser.add_argument(
+        '--version',
+        action='version',
+        version=__version__
+    )
+    parser.add_argument(
+        '--traceback',
+        action='store_true',
+        help="show traceback for exceptions.",
+    )
+    parsed_args = parser.parse_args(args)
+
+    if parsed_args.verbose == 0:
+        log_level = logging.WARNING
+    elif parsed_args.verbose == 1:
+        log_level = logging.INFO
+    else:
+        log_level = logging.DEBUG
+
+    logging.basicConfig(format="%(levelname)s: %(message)s", level=log_level)
+    for handler in logging.root.handlers:
+        handler.addFilter(DuplicateMessageFilter())
+
+    if (parsed_args.output_paths and
+            len(parsed_args.font_paths) != len(parsed_args.output_paths)):
+        parser.error("number of input and output fonts differ")
+
+    if len(parsed_args.font_paths) != len(set(parsed_args.font_paths)):
+        parser.error("found duplicate font inputs")
+
+    options = ReportOptions(parsed_args)
+
+    options.font_format = _validate_font_paths(parsed_args.font_paths, parser)
+
+    if parsed_args.glyphs_to_hint:
+        options.glyphList = _process_glyph_list_arg(
+            parsed_args.glyphs_to_hint, options.nameAliases)
+    elif parsed_args.glyphs_to_not_hint:
+        options.excludeGlyphList = True
+        options.glyphList = _process_glyph_list_arg(
+            parsed_args.glyphs_to_not_hint, options.nameAliases)
+    elif parsed_args.glyphs_to_hint_file:
+        options.glyphList = _process_glyph_list_arg(
+            _read_txt_file(parsed_args.glyphs_to_hint_file),
+            options.nameAliases)
+    elif parsed_args.glyphs_to_not_hint_file:
+        options.excludeGlyphList = True
+        options.glyphList = _process_glyph_list_arg(
+            _read_txt_file(parsed_args.glyphs_to_not_hint_file),
+            options.nameAliases)
+
+    return options, parsed_args
+
+
+def stemhist(args=None):
+    options, pargs = get_stemhist_options(args)
+
+    try:
+        hintFiles(options)
+    except Exception as ex:
+        if pargs.traceback:
+            raise
+        logging.error(ex)
+        return 1
 
 
 if __name__ == '__main__':
